/** **************************************************************************
 * Copyright (C) 2002 David Lucardi <davidlucardi@aol.com>
 * Copyright (C) 2005 Eric Seigne <eric.seigne@ryxeo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * ***************************************************************************
 * File  : kidistb.c
 * Author  : David LUCARDI
 *           mailto:davidlucardi@aol.com
 *           http://www.abuledu.org/leterrier/
 * Date    : 30/10/2002
 * Licence : GNU/GPL Version 2
 *
 * Description:
 * ------------
 *
 * @version    $Id: kidistb.c,v 1.4 2005/07/30 06:27:32 erics Exp $
 * @author     David Lucardi
 * @project   
 * @copyright  David Lucardi 2002
 *
 *   Thanks to : 
 *   Siome Klein Goldenstein (tcl integration)
 *   John Gauch, Edu Metz (zooming algorithms)
 *
 * ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <tcl.h>
#include <tk.h>
#include "kidistb.h"


int Kidistb_Init (Tcl_Interp *interp)
{
  Tcl_CreateCommand (interp, "CZoom", Zoom_czoom, (ClientData *) NULL, (Tcl_CmdDeleteProc *) NULL);
  return TCL_OK;
}

int Zoom_czoom (ClientData cd, Tcl_Interp *interp, int argc, char **argv)
{
  Tk_PhotoHandle      tozoom;
  Tk_PhotoImageBlock  blorig, blnew;
  
  double xZoom;
  double yZoom;
  float OldX, OldY;
  float PartX, PartY;
  int Xdim, Ydim, x, y;
  int var1,var2;
  Tcl_GetDouble (interp, argv[2], &xZoom);
  Tcl_GetDouble (interp, argv[3], &yZoom);
  tozoom = Tk_FindPhoto (interp, argv[1]);
  Tk_PhotoGetSize  (tozoom, &Xdim, &Ydim);
  Tk_PhotoGetImage (tozoom, &blorig);
  blnew.width  = (int)(Xdim*xZoom);
  blnew.height = (int)(Ydim*yZoom);  
  
  if (yZoom <1)
    { 
      var2 = Ydim;
    }
  else
    {
      var2 = blnew.height;
    }
  
  if (xZoom <1)
    { 
      var1 = Xdim;
    }
  else
    {
      var1 = blnew.width;
    }
  
  
  ////////////////////////////////////////////////////////////:
  // blnew.pitch  = var1;
  //blnew.pixelSize = 1;
  //blnew.offset[0] = 0;
  //blnew.offset[1] = var1*var2;
  //blnew.offset[2] = 2*var1*var2;
  //blnew.offset[3] = 3*var1*var2;
  //blnew.offset[4] = 4*var1*var2;
  ////////////////////////////////////////////////////////////::
  blnew.pitch  = 4*blnew.width;
  blnew.pixelSize = 4;
  blnew.offset[0] = 0;
  blnew.offset[1] = 1;
  blnew.offset[2] = 2;
  blnew.offset[3] = 3;
  //blnew.offset[4] = 4;
  
  if ( (blnew.pixelPtr = (unsigned char*) calloc (4*blnew.width*blnew.height, sizeof (unsigned char)))== NULL)
    {
      //fprintf (stderr,"Error in memory allocation\n");
      exit (1);
    }
  
  
  for (y=0; y< blnew.height; y++)
    {
      OldY=y/yZoom;	 
      for (x=0; x<blnew.width; x++)
	{
	  OldX =  x/xZoom;
	  
	  if ((OldX > 0) && (OldY > 0) && (OldX < var1 - 1) && (OldY < var2 - 1))
	    {
	      /* Nearest neighbor sampling */
	      RED(&blnew, x, y) = RED(&blorig, (int) (OldX + 0.5), (int) (OldY + 0.5));
	      GREEN(&blnew, x, y) = GREEN(&blorig, (int) (OldX + 0.5), (int) (OldY + 0.5));
	      BLUE(&blnew, x, y) = BLUE(&blorig, (int) (OldX + 0.5), (int) (OldY + 0.5));
	      ALPHA(&blnew, x, y) = ALPHA(&blorig, (int) (OldX + 0.5), (int) (OldY + 0.5));
	      
	      /*bilinear :  not used*/
	      /////////////////////////////////////:
	      //PartX = OldX - (int) OldX;
	      //PartY = OldY - (int) OldY;
	      //RED(&blnew, x,y)= (unsigned char)((1 - PartX) * (1 - PartY) * RED(&blorig, (int) OldX, (int) OldY) + (1 - PartX) * (PartY) * RED(&blorig, (int) OldX + 1, (int) OldY) + (PartX) * (1 - PartY) * RED(&blorig, (int) OldX, (int) OldY + 1) + (PartX) * (PartY) * RED(&blorig, (int) OldX + 1, (int) OldY + 1));
	      //GREEN(&blnew, x,y)= (unsigned char)((1 - PartX) * (1 - PartY) * GREEN(&blorig, (int) OldX, (int) OldY) + (1 - PartX) * (PartY) * GREEN(&blorig, (int) OldX + 1, (int) OldY) + (PartX) * (1 - PartY) * GREEN(&blorig, (int) OldX, (int) OldY + 1) + (PartX) * (PartY) * GREEN(&blorig, (int) OldX + 1, (int) OldY + 1));
	      //BLUE(&blnew, x,y)= (unsigned char)((1 - PartX) * (1 - PartY) * BLUE(&blorig, (int) OldX, (int) OldY) + (1 - PartX) * (PartY) * BLUE(&blorig, (int) OldX + 1, (int) OldY) + (PartX) * (1 - PartY) * BLUE(&blorig, (int) OldX, (int) OldY + 1) + (PartX) * (PartY) * BLUE(&blorig, (int) OldX + 1, (int) OldY + 1));
	      //ALPHA(&blnew, x,y)= (unsigned char)((1 - PartX) * (1 - PartY) * ALPHA(&blorig, (int) OldX, (int) OldY) + (1 - PartX) * (PartY) * ALPHA(&blorig, (int) OldX + 1, (int) OldY) + (PartX) * (1 - PartY) * ALPHA(&blorig, (int) OldX, (int) OldY + 1) + (PartX) * (PartY) * ALPHA(&blorig, (int) OldX + 1, (int) OldY+ 1));
	      ////////////////////////////////////////
	    }
	  else
	    {
	      RED(&blnew, x, y) = GREEN(&blnew, x, y) = BLUE(&blnew, x, y)= ALPHA(&blnew, x, y)=0;
	    }
	}
    }
  
  Tk_PhotoBlank(tozoom);
  Tk_PhotoSetSize(tozoom,blnew.width - (int)xZoom,blnew.height - (int)yZoom);
  //erics, tcl 8.4 et 8.3 n'ont pas la meme fonction !
  //#if TCL_MAJOR_VERSION == 8 &&  TCL_MINOR_VERSION == 4
  //#elif TCL_MAJOR_VERSION == 8 &&  TCL_MINOR_VERSION == 3
#ifdef HAVE_LIBTCL8_4
  Tk_PhotoPutBlock (tozoom,&blnew,0,0,blnew.width - (int)xZoom,blnew.height - (int)yZoom, 0);
#elifdef HAVE_LIBTCL8_3
  Tk_PhotoPutBlock (tozoom,&blnew,0,0,blnew.width - (int)xZoom,blnew.height - (int)yZoom);
#endif
  Tk_DitherPhoto(tozoom, 0, 0, blnew.width - (int)xZoom, blnew.height - (int)yZoom);
  
  free (blnew.pixelPtr);
  return TCL_OK; 
}

